package d11;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.util.ArrayList;
import java.util.Scanner;

public class TehtavaT3 {
    public static class Valuutta {
        private String valuutta;
        private double kurssi;

        public Valuutta(String v, double k) {
            this.valuutta = v;
            this.kurssi = k;
        }

        public String getValuutta() {
            return valuutta;
        }

        /**
         * Muuttaa annetun raha määrän valuutan kurssin mukaiseksi
         * @param maara rahan määrä
         * @return rahan määrä markkoina
         */
        public double laskeMuutos(double maara) {
            return maara * kurssi;
        }
        public String toString() {
            return String.format("%6s %2.5f", valuutta, kurssi);
        }
    }

    /**
     * @param args Ensimmäinen argumentti luettavan tiedoston nimi
     */
    public static void main(String[] args) {
        String tiedosto = "valuutat.txt";
        if (args.length > 0) {
            tiedosto = args[0];
        }
        ArrayList<Valuutta> valuutat = lueValuutat(tiedosto);
        System.out.println("val.\tkur.");
        System.out.println("------------");
        for (Valuutta v: valuutat) {
            System.out.println(v);
        }
        Scanner scanner = new Scanner(System.in);
        while (scanner.hasNextLine()) {
            System.out.print("Määrä ja valuutta>");
            String userInput = scanner.nextLine();
            if (userInput.equals("loppu")) {
                System.out.println("Kiitos!");
                break;
            }
            String[] osat = userInput.split(" ");
            try {
                double maara = Double.parseDouble(osat[0]);
                String valuutta = osat[1].toLowerCase();
                double vaihdos = vaihdaValuutta(maara, valuutta, valuutat);
                System.out.printf("%.2f %s on %.2f mk\n", maara, valuutta, vaihdos);
            } catch (NumberFormatException e) {
                System.out.println("Virheellinen syöte");
            }
        }
    }

    /**
     * Lukee annetun tiedoston ja palauttaa niistä taulukollisen valuuttoja
     * @param tiedostoNimi tiedoston nimi
     * @return taulukollinen valuuttoja
     */
    public static ArrayList<Valuutta> lueValuutat(String tiedostoNimi) {
        ArrayList<Valuutta> valuutat = new ArrayList<>();
        try (Scanner scanner = new Scanner(new FileInputStream(tiedostoNimi))) {
            while (scanner.hasNext()) {
                String rivi = scanner.nextLine();
                String[] osat = rivi.split(" ");
                double kurssi = Double.parseDouble( osat[0]);
                String valuutta = osat[1];
                valuutat.add(new Valuutta(valuutta, kurssi));
            }
        } catch (FileNotFoundException e) {
            throw new RuntimeException(e);
        }
        return valuutat;
    }

    /**
     * Vaihtaa annettun raha määrä annettua valuuttaa markoiksi
     * @param maara raha määrä
     * @param valuutta annettu valuutta
     * @param valuutat Vaihdettavat valuutat, joista annettun valuutan pittää löytyä
     * @return palauttaa rahamäärän markkoina
     * @example
     * <pre name="test">
     * Valuutta[] valuutat = {
     *         new Valuutta("mk", 1.0),
     *         new Valuutta("$", 5.7),
     *         new Valuutta("EUROA", 5.94573),
     *         new Valuutta("SKr", 0.6),
     *         };
     * vaihdaValuutta(5, "skr", valuutat) ~~~ 3.0;
     * vaihdaValuutta(5, "$", valuutat) ~~~ 28.5;
     * </pre>
     */
    public static double vaihdaValuutta(double maara, String valuutta, ArrayList<Valuutta> valuutat) {
        double summa = 0.0;
        for (Valuutta v: valuutat) {
            if (v.getValuutta().toLowerCase().equals(valuutta)) {
                summa = v.laskeMuutos(maara);
            }
        }
        return summa;
    }
}

