package d12;


import fi.jyu.mit.ohj2.Syotto;

import java.util.ArrayList;
import java.util.List;

/**
 *
 */
public class Komentorivi {
// #DYNAMICIMPORT
    /**
     * Suorittaa yhden komennon.
     */
    public interface KomentoRajapinta {
        String suorita(String parametrit);
    }


    public static class Ynnaa implements KomentoRajapinta {
        /**
         * @param parametrit teksti jono muotoa "komento parametrit" esim "ynnaa 1 2 3"
         * @return jos parametri löytyy, muuttaa sana isoksi
         * @example <pre name="test">
         * Ynnaa y = new Ynnaa();
         * y.suorita("Ynnaa 2 3 4") === "Tulos on 9."
         * y.suorita("Ynnaa") === "Tulos on 0."
         * </pre>
         */
        @Override
        public String suorita(String parametrit) {
            String osat[] = parametrit.split(" ");
            int summa = 0;
            if (osat.length > 1) {
                for(String s : osat) {
                    try {
                        summa += Integer.parseInt(s);
                    } catch (NumberFormatException e) {
                        continue;
                    }
                }
            }
            return String.format("Tulos on %d.", summa);
        }
    }

    public static class Apua implements KomentoRajapinta {

        private String tiedosto; //tiedosto josta apua löytyy etsivälle
        public Apua(String t) {
            this.tiedosto = t;
        }

        @Override
        public String suorita(String parametrit) {
            return "";
        }
    }

    public static class Palindromiko implements KomentoRajapinta {
        /**
         * @param parametrit teksti jono muotoa "komento parametrit" esim "palindromiko saippuakivikauppias"
         * @return jos parametri löytyy, muuttaa sana isoksi
         * @example <pre name="test">
         * Palindromiko p = new Palindromiko();
         * p.suorita("Palindromiko saippuakivikauppias") === "Sana 'saippuakivikauppias' on palindromi!"
         * p.suorita("Palindromiko kana") === "Sana kana ei ole palindromi"
         * </pre>
         */
        @Override
        public String suorita(String parametrit) {
            String[] osat = parametrit.split(" ");
            if (osat.length > 1) {
                String sana = osat[1];
                if (sana.equals(new StringBuilder(sana).reverse().toString())) {
                    return String.format("Sana '%s' on palindromi!",sana);
                } else {
                    return String.format("Sana %s ei ole palindromi", sana);
                }
            }
            return "";
        }
    }

    public static class Isoksi implements KomentoRajapinta {

        /**
         * @param parametrit teksti jono muotoa "komento parametrit"
         * @return jos parametri löytyy, muuttaa sana isoksi
         * @example <pre name="test">
         * Isoksi i = new Isoksi();
         * i.suorita("isoksi kana") === "kana on isona KANA"
         * </pre>
         */
        @Override
        public String suorita(String parametrit) {
            String[] osat = parametrit.split(" ");
            if (osat.length > 1) {
                return String.format("%s on isona %s", osat[1], osat[1].toUpperCase());
            }
            return "";
        }
    }


    /**
     * Komennon nimi ja vastaava "funktio".
     */
    public static class Komento {

        private String nimi;
        private KomentoRajapinta komento;

        public Komento(String nimi, KomentoRajapinta komento) {
            this.nimi = nimi;
            this.komento = komento;
        }

        public String suoritaKomento(String parametrit) {
            return komento.suorita(parametrit);
        }

    }


    /**
     * Lista komennoista ja metodit etsimiseksi ja suorittamiseksi.
     */
    public static class Komennot {
        private static List<Komento> komennot = new ArrayList<Komento>();

        public void add(Komento k) {
            komennot.add(k);
        }

        /**
         * Kutsuu komennon suorita metodia ja palauttaa tuloksen jos komento löytyy
         * @param parametrit teksti muodossa "komento parametrit"
         * @return suorita metodin tulos tai "" jos ei komentoa löydy
         */
        public String tulkitse(String parametrit) {
            String komennonNimi = parametrit.split(" ")[0];
            for (Komento k: komennot) {
                if (k.nimi.contains(komennonNimi)) {
                    return k.suoritaKomento(parametrit);
                }
            }
            return String.format("En tunne komentoa %s!", komennonNimi);
        }
    }


    /**
     * Testipääohjelma
     *
     * @param args ei käytössä
     */
    public static void main(String[] args) {
        Komennot komennot = new Komennot();
        Apua apua = new Apua("komento.txt");
        komennot.add(new Komento("?", apua));
        komennot.add(new Komento("apua", apua));
        komennot.add(new Komento("+", new Ynnaa()));
        komennot.add(new Komento("ynnää", new Ynnaa()));
        komennot.add(new Komento("isoksi", new Isoksi()));
        komennot.add(new Komento("palindromiko", new Palindromiko()));

        String s;

        while (true) {
            s = Syotto.kysy("Anna komento");
            if ("".equals(s))
                break;
            String tulos = komennot.tulkitse(s);
            System.out.println(tulos);
        }
    }
}