package d12;
import fi.jyu.mit.ohj2.Syotto;

import java.io.*;
import java.net.URI;
import java.util.Scanner;
import java.util.SortedMap;
import java.util.TreeMap;

/**
 * Ohjelmalla vaihdetaan valuuttoja
 * Versioon 1.1 lisätty Valuutat -luokan rajapintaa
 * uusia metodeja, mm. getVaihdettuMaara, getValuutat
 * @author Vesa Lappalainen
 * @version 1.0, 15.03.2003
 * @version 1.1, 09.04.2003
 * @version 1.2, 30.03.2025
 */
public class ValuuttaMuunnos {
    // #DYNAMICIMPORT

    /**
     * Muunnetaan valuuttaa
     * @param args ei käytössä
     * @throws IOException jos jokin menee pieleen tiedoston luvussa
     */
    public static void main(String[] args) throws IOException {
        Valuutat valuutat = new Valuutat();
        /*
            valuutat.lisaa(1.0,"mk");
            valuutat.lisaa(5.7,"$");
            valuutat.lisaa(5.9,"EUROA");
            valuutat.lisaa(0.6,"SKr");
        */
        if (!valuutat.lue("valuutat.txt")) return;

        ValuuttaNaytto naytto = new ValuuttaNaytto(valuutat);

        while (naytto.kysy()) {
            naytto.tulosta();
        }
    }

    /**
     * Luokka valuuttojen tallentamiselle
     * @author Vesa Lappalainen
     * @version 1.0, 15.03.2003
     */
    public static class Valuutat {
        private final SortedMap<String, Double> alkiot = new TreeMap<>();
        private String ensimmainenValuutta = null;

        /**
         * Lisätään tietorakenteeseen uusi valuutta.
         * @param kerroin lisättävän valuutan kerroin
         * @param valuutta listtävän valuutan yksikkö
         * @example
         * <pre name="test">
         * Valuutat valuutat = new Valuutat();
         * valuutat.lisaa(1,"EUROA");
         * valuutat.lisaa(10,"SKr");
         * valuutat.lisaa(1.6,"$");
         * valuutat.ekanValuutannimi() === "EUROA";
         * valuutat.getKerroin("$") ~~~ 1.6;
         * valuutat.getValuutannimi("SKr") === "SKr";
         * </pre>
         */
        public void lisaa(double kerroin, String valuutta) {
            if (ensimmainenValuutta == null) ensimmainenValuutta = valuutta;
            alkiot.put(valuutta, kerroin);
        }


        /**
         * Palautetaan ensimmäisen tietorakenteessa olevan
         * valuutan yksikkö.
         * @return ensimmäisen valuutan yksikkö
         */
        public String ekanValuutannimi() {
            if (alkiot.isEmpty()) return "";
            return ensimmainenValuutta;
        }



        /**
         * Etsitään hakujonoa vastaavan valuutan kerroin.
         * @param valuutannimi etsittävän valuutan yksikkö
         * @return valuutan kerroin tai 1.0 jo ei löydy.
         */
        public double getKerroin(String valuutannimi) {
            double val = alkiot.getOrDefault(valuutannimi, 0.0);
            if (val == 0.0) return 1.0;
            return val;
        }


        /**
         * Etsitään hakujonoa vastaavan valuutan yksikkö
         * @param valuutannimi etsittävän valuutan yksikkö
         * @return valuutan nimi tai "" jos ei löydy
         */
        public String getValuutannimi(String valuutannimi) {
            boolean val = alkiot.containsKey(valuutannimi);
            if (!val) return "";
            return valuutannimi;
        }


        /**
         * Etsitään hakujonoa vastaavan valuutan yksikkö
         * @param valuutta etsittävä valuutan hakujono valuutassa
         * @return valuutan kerroin tai 1.0 jo ei löydy.
         */
        public double getValuutta(String valuutta) {
            return alkiot.getOrDefault(valuutta, 1.0);
        }


        /**
         * Luetaan valuutat tiedostosta tai www:stä.
         * Jos tiedoston nimi alkaa http-luetaan www:stä
         * @param nimi luettavan valuuttatiedoston nimi
         * @return true jos lukeminen onnistuu.
         * @throws IOException jos jokin menee pieleen tiedoston käsittelyssä.
         */
        public boolean lue(String nimi) throws IOException {
            try (Scanner fi = nimi.startsWith("http") ?
                    new Scanner(new URI(nimi).toURL().openConnection().getInputStream()) :
                    new Scanner(new File(nimi))) {
                while (fi.hasNext()) {
                    String[] osat = fi.nextLine().split(" ");
                    lisaa(Double.parseDouble(osat[0]), osat[1]);
                }
                return true;
            } catch (Exception e) {
                throw new IOException(e);
            }
        }


        /**
         * Palautetaan haettavaa valuuttaa vastaava summa kantavaluutassa.
         * @param nimi  haettavan valuutan hakujono
         * @param maara rahamäärä joka muutetaan kantavaluuttaan.
         * @return rahamäärä muutettuna kantavaluuttaan.
         */
        public double getVaihdettuMaara(String nimi, double maara) {
            return getKerroin(nimi) * maara;
        }


        /**
         * Palautetaan valuutassa olevaa hakujonoa vastaava valuutta-olio
         * kantavaluuttana.
         * @param val valuutta jossa on hakujono ja määrä joka muutetaan kantavaluuttaan.
         * @return tiedot kantavaluuttana
         * @example
         * <pre name="test">
         * Valuutat valuutat = new Valuutat();
         * valuutat.lisaa(1,"e");
         * valuutat.lisaa(10,"SKr");
         * valuutat.lisaa(1.6,"$");
         * valuutat.getVaihdettu("SKr",3.0) ~~~ 30.0;
         * valuutat.getVaihdettu("$", 3.0) ~~~ 4.80;
         * </pre>
         */
        public double getVaihdettu(String val, double maara) {
            return getVaihdettuMaara(val, maara);
        }


        /**
         * Palautetaan merkkijonotaulukkona kaikkien valuuttojen nimet
         * @return valuuttojen nimet merkkijonotaulukossa.
         * @example
         * <pre name="test">
         * Valuutat valuutat = new Valuutat();
         * valuutat.lisaa(1,"e");
         * valuutat.lisaa(10,"SKr");
         * valuutat.lisaa(1.6,"$");
         * String nimet[] = valuutat.getValuuttojenNimet();
         * nimet.length === 3;
         * nimet[2] === "e";
         * nimet[1] === "SKr";
         * nimet[0] === "$";
         * </pre>
         */
        public String[] getValuuttojenNimet() {
            String[] nimet = new String[alkiot.size()];
            int i = 0;
            for (String valuutta: alkiot.keySet()) nimet[i++] = valuutta;
            return nimet;
        }

    }

    /**
     * Näyttö-luokka valuuttojen käyttämiseksi konsolisovelluksesta.
     * @author Vesa Lappalainen
     * @version 1.0, 15.03.2003
     *
     * @example
     * <pre name="test">
     * @example
     * <pre name="test">
     * #import fi.jyu.mit.ohj2.Suuntaaja;
     *
     * Valuutat valuutat = new Valuutat();
     * valuutat.lisaa(1,"EUROA");
     * valuutat.lisaa(10,"SKr");
     * valuutat.lisaa(1.6,"$");
     * ValuuttaNaytto naytto = new ValuuttaNaytto(valuutat);
     *
     * Suuntaaja.StringInput si = new Suuntaaja.StringInput("");
     * Suuntaaja.StringOutput so = new Suuntaaja.StringOutput();
     *
     * si.input("");       naytto.kysy() === false;
     * si.input("loppu");  naytto.kysy() === false;
     * so.reset();
     * si.input("3 SKr");    naytto.kysy() === true;
     * so.reset(); naytto.tulosta(); so.ero("3.00 SKr on 30.00 EUROA\n") === null;
     * si.input("3 $");    naytto.kysy() === true;
     * so.reset(); naytto.tulosta(); so.ero("3.00 $ on 4.80 EUROA\n") === null;
     * si.input("2");      naytto.kysy() === true;
     * so.reset(); naytto.tulosta(); so.ero("2.00 $ on 3.20 EUROA\n") === null;
     * si.input("k lati"); naytto.kysy() === true; // Yksikköä ei löydy
     * so.reset(); naytto.tulosta(); so.ero("2.00  on 2.00 EUROA\n") === null;
     * si.input("");       naytto.kysy() === false;
     *
     * si.palauta(); so.palauta();
     *
     * </pre>
     * </pre>
     */
    public static class ValuuttaNaytto {
        private final Valuutat valuutat;
        private String valuutta = "";
        private double maara = 0.0;


        /**
         * Alustetaan näyttö käyttämään valuuttataulukkoa.
         * @param valuutat valuuttataulukko jota käytetään.
         */
        public ValuuttaNaytto(Valuutat valuutat) {
            this.valuutat = valuutat;
        }


        /**
         * Kysytään valuutan tietoja.
         * @return true jos kysely onnistui.
         */
        public boolean kysy() {
            String jono = Syotto.kysy("Määrä ja valuutta");

            if ("".equals(jono)) return false;
            if ("loppu".equals(jono)) return false;

            String[] osat = jono.split(" ");
            if (osat.length > 0) {
                try {
                    maara = Double.parseDouble(osat[0]);
                } catch (NumberFormatException e) {
                    valuutta = "";
                    return true;
                }
            }
            if (osat.length > 1) {
                valuutta = osat[1];
            }
            return true;
        }


        /**
         * Tulostetaan kysytyn valuutan tiedot.
         */
        public void tulosta() {
            System.out.printf("%.2f %s on %.2f %s%n", maara, valuutta, maara * valuutat.getValuutta(valuutta), valuutat.ekanValuutannimi());
        }

    }
}
