package d3;

import fi.jyu.mit.ohj2.Mjonot;

public class Main {
    public static void main(String[] args) {
        if (poistaEsiintymat("Catcat", "at").equals("Cc")) {
            System.out.println("Catcat => Cc");
        }
        if (poistaEsiintymat("Paatti", "at").equals("Pi")) {
            System.out.println("Paatti => Pi");
        }
        if (poistaEsiintymat("elielin aukio", "eli").equals("n aukio")) {
            System.out.println("elielin aukio => n aukio");
        }
        if (poistaEsiintymat("kaapelitehdas", "").equals("kaapelitehdas")) {
            System.out.println("kaapelitehdas => kaapelitehdas");
        }
    }

    /**
     * @param jono  Merkkijono
     * @param maski Merkkijono jota vasten tarkistetaan jono. Maski hyväksyy erikoismerkin
     *              "?", joka vastaa mitä tahansa kirjainta.
     * @return onko jonot samat
     * @example <pre name="test">
     * onkoSamatKysymysmerkilla("Kissa", "K?ssa") === true
     * onkoSamatKysymysmerkilla("Kissa", "Kissa") === true
     * onkoSamatKysymysmerkilla("Kissa", "Kassa") === false
     * onkoSamatKysymysmerkilla("", "K") === false
     * </pre>
     */
    public static boolean onkoSamatKysymysmerkilla(String jono, String maski) {
        boolean onkoSamat = true;
        if (jono.length() != maski.length()) {
            onkoSamat = false;
        } else {
            for (int i = 0; i < jono.length(); i++) {
                if (maski.charAt(i) != '?' && jono.charAt(i) != maski.charAt(i)) {
                    onkoSamat = false;
                    break;
                }
            }
        }
        return onkoSamat;
    }

    /**
     * @param jono        Merkkijono josta poistetaan esiintymät
     * @param poistettava Merkkijonosta poistettava esiintymä
     * @return palautaa merkkijonon, josta on poistettu kaikki poistettavat esiintymät
     * @example <pre name="test">
     * poistaEsiintymat("Catcat", "at") === "Cc"
     * poistaEsiintymat("Paatti", "at") === "Pi"
     * poistaEsiintymat("Puatit", "at") === "Puit"
     * poistaEsiintymat("elielin aukio", "eli") === "n aukio"
     * poistaEsiintymat("a", "at") === "a"
     * poistaEsiintymat("kaapelitehdas", "") === "kaapelitehdas"
     * </pre>
     */
    public static String poistaEsiintymat(String jono, String poistettava) {
        /* Jos molemmat jompikumpi on tyhjä, palautetaan sellaisenaan */
        if (jono.isEmpty() || poistettava.isEmpty()) {
            return jono;
        }
        StringBuilder jonoStringBuilder = new StringBuilder(jono);
        boolean eiPoistettavaa = false;
        while (!eiPoistettavaa) {
            boolean poistettu = false;
            int poistettavanJonoPituus = poistettava.length();
            /* Jos jäljellä oleva jono pienempi kuin poistettava jono */
            if (jonoStringBuilder.length() < poistettava.length()) {
                break;
            } else {
                for (int i = 0; i <= jonoStringBuilder.length() - poistettavanJonoPituus; i++) {
                    if (jonoStringBuilder.substring(i, i + poistettavanJonoPituus).equals(poistettava)) {
                        jonoStringBuilder.delete(i, i + poistettavanJonoPituus);
                        poistettu = true;
                        break;
                    }
                }
            }
            eiPoistettavaa = !poistettu;
        }
        return jonoStringBuilder.toString();
    }

    /**
     * Vaihtaa annetun merkkijonon alku- ja loppuosat keskenään.
     *
     * @param merkkijono Merkkijono jonka osat vaihdetaan
     * @param etsiTakaperin Määrittää kummasta suunnasta merkkijonoa etsitään erotin ' ',
     * @return merkkijonon osat vaihdettuna
     * @example <pre name="test">
     *   vaihdaAlkuLoppu("Aku Ankka", false) === "Ankka Aku";
     *   vaihdaAlkuLoppu("Aku Teppo Ankka", true) === "Ankka Aku Teppo";
     *   vaihdaAlkuLoppu("Aku Teppo Ankka", false) === "Teppo Ankka Aku";
     * </pre>
     */
    public static String vaihdaAlkuLoppu(String merkkijono, boolean etsiTakaperin) {
        StringBuilder loppu = new StringBuilder(merkkijono);
        String alku = Mjonot.erota(loppu, ' ', etsiTakaperin);
        if (loppu.isEmpty()) {
            return alku;
        }
        return loppu + " " + alku;
    }


    /**
     * Vaihtaa etunimen ja sukunimen keskenään,
     * Jukka Pekka Palo => Palo Jukka Pekka
     *
     * @param nimi Etunimi Sukunimi jota vaihdetaan
     * @return Sukunimi Etunimi
     * @example <pre name="test">
     *   vaihdaEtuSuku("") === ""
     *   vaihdaEtuSuku(" ") === ""
     *   vaihdaEtuSuku("Pele") === "Pele"
     *   vaihdaEtuSuku("Aku Ankka") === "Ankka Aku"
     *   vaihdaEtuSuku("Jukka Pekka Palo") === "Palo Jukka Pekka"
     * </pre>
     */
    public static String vaihdaEtuSuku(String nimi) {
        return vaihdaAlkuLoppu(nimi, true);
    }


    /**
     * Vaihtaa sukunimen ja etunimen keskenään,
     * Palo Jukka Pekka => Jukka Pekka Palo
     *
     * @param nimi Sukunimi Etunimi jota vaihdetaan
     * @return Etunimi Sukunimi
     * @example <pre name="test">
     *   vaihdaSukuEtu("") === ""
     *   vaihdaSukuEtu("Pele") === "Pele"
     *   vaihdaSukuEtu("Ankka Aku") === "Aku Ankka"
     *   vaihdaSukuEtu("Palo Jukka Pekka") === "Jukka Pekka Palo"
     * </pre>
     */
    public static String vaihdaSukuEtu(String nimi) {
        return vaihdaAlkuLoppu(nimi, false);
    }
}