package d3;

import fi.jyu.mit.ohj2.Mjonot;

/**
 * @example <pre name="test">
 * Pvm pvm = new Pvm(5,5, 2025);
 * pvm.toString() === "5.5.2025"
 * </pre>
 */
public class Pvm {

    private int pv = 1;
    private int kk = 1;
    private int vv = 1900;

    /**
     * Alustaa päivämäärä olino
     * @param pv Päivä
     * @param kk Kuukausi
     * @param vv Vuosi
     *
     * @example <pre name="test">
     * new Pvm(5, 5, 2025).toString() === "5.5.2025"
     * </pre>
     */
    public Pvm(int pv, int kk, int vv) {
        this.pv = pv;
        this.kk = kk;
        this.vv = vv;
    }

    /** Alustetaan kaikki attribuutit oletusarvoon */
    public Pvm() {
        this(0, 0, 0);
    }

    public Pvm(String pvmJono) {
        this.parse(pvmJono);
    }

    /**
     * @return päivämäärän merkkijonona muodossa 1.1.1990
     * @example <pre name="test">
     * new Pvm(5, 5, 2025).toString() === "5.5.2025"
     * </pre>
     */
    @Override
    public String toString() {
        return String.format("%d.%d.%d", pv, kk, vv);
    }

    /**
     * @return palauttaa Pvm otukselle asetetun päivän
     *
     * @example <pre name="test">
     * Pvm pv = new Pvm("3.2.2020");
     * pv.getPv() === 3
     * </pre>
     */
    public int getPv() {
        return this.pv;
    }
    /**
     * @return palauttaa Pvm otukselle asetetun kuukauden
     *
     * @example <pre name="test">
     * Pvm pv = new Pvm("3.2.2020");
     * pv.getKk() === 2
     * </pre>
     */
    public int getKk() {
        return this.kk;
    }
    /**
     * @return palauttaa Pvm otukselle asetetun vuoden
     *
     * @example <pre name="test">
     * Pvm pv = new Pvm("3.2.2020");
     * pv.getVv() === 2020
     * </pre>
     */
    public int getVv() {
        return this.vv;
    }

    /**
     * Alustetaan päivämäärä. 0-arvot eivät muuta vastaavaa attribuuttia
     * TODO: oikeellisuustarkitukset
     * @param ipv päivän alustus
     * @param ikk kuukauden alustus
     * @param iivv vuoden alustus
     */
    public final void alusta(final int ipv, final int ikk, final int iivv) {
        if (ipv > 0)        this.pv = ipv;
        if (ikk > 0)        this.kk = ikk;
        if (iivv > 0)        this.vv = iivv;
        if (this.vv < 50)  this.vv += 2000;
        if (this.vv < 100) this.vv += 1900;
    }

    /**
     * Alustetaan päivän arvon merkkijonosta
     * @param s jono josta pvm otetaan
     * @example
     * <pre name="test">
     * Pvm pvm = new Pvm();
     * pvm.parse("11.05.02");  pvm.toString() === "11.5.2002";
     * pvm.parse("11.05.97");  pvm.toString() === "11.5.1997";
     * pvm.parse("17.3");      pvm.toString() === "17.3.1997";
     * pvm.parse("19");        pvm.toString() === "19.3.1997";
     * </pre>
     */
    public void parse(String s) {
        StringBuffer sb = new StringBuffer(s);
        int p = Mjonot.erota(sb, '.', 0);
        int k = Mjonot.erota(sb, '.', 0);
        int v = Mjonot.erota(sb, '.', 0);
        alusta(p, k, v);
    }
}
