package d5;

import java.io.OutputStream;
import java.io.PrintStream;

/**
 * LinjaAuto luokka luo linjaAuto otuksen, jolla voidaan ylläpitää matkustajen määrää
 */
public class LinjaAuto {
    private int matkustajat, tilaa, paikat;

    /**
     * @param paikat määrittää linjaAuton kokonaispaikka
     * @example <pre name="test">
     * LinjaAuto la = new LinjaAuto(50);
     * la.getPaikat() === 50
     * la.getMatkustajat() === 0
     * la.getTilaa() === 50
     */
    public LinjaAuto(int paikat) {
        this.paikat = paikat;
        this.matkustajat = 0;
        this.tilaa = paikat;
    }

    /**
     * @return LinjaAuton matkustajat
     */
    public int getMatkustajat() {
        return matkustajat;
    }

    /**
     * @return LinjaAuton vapaat paikat
     */
    public int getTilaa() {
        return tilaa;
    }

    /**
     * @return LinjaAuton paikat
     */
    public int getPaikat() {
        return paikat;
    }

    /**
     * @param vahennys Vähentää linja-auton matkustajia. Jos koitetaan vähentää enemmän kuin matkustajia on
     * asetetaan matkustajien määrä 0 ja vapaat == paikat ja matkustajan määrä
     * @return matkustajan määrä
     *
     * @example <pre name="test">
     * LinjaAuto la = new LinjaAuto(50);
     * la.lisaa(5);
     * la.getTilaa() === 45
     * la.getMatkustajat() === 5
     * la.vahenna(5);
     * la.getTilaa() === 50
     * la.getMatkustajat() === 0
     * </pre>
     */
    public int vahenna(int vahennys) {
        matkustajat -= vahennys;
        tilaa += vahennys;
        int palautus = paikat - tilaa;
        if (matkustajat <= 0) {
            matkustajat = 0;
            tilaa = paikat;
        }
        return palautus;
    }

    /**
     * @param lisays Lisää linja-auton matkustajia. Jos vapaita on vähemmän kuin lisättäviä paikkoja
     * lisätään matkustajia, niin monta kuin on vapaita paikkoja.
     * @return ylimenevät
     * @example <pre name="test">
     * LinjaAuto la = new LinjaAuto(50);
     * la.lisaa(5);
     * la.getTilaa() === 45
     * la.getMatkustajat() === 5
     * </pre>
     */
    public int lisaa(int lisays) {
        int palautus = 0;
        if (lisays > tilaa) {
            palautus = lisays - tilaa;
            tilaa = 0;
            matkustajat = paikat;
        } else {
            matkustajat += lisays;
            tilaa -= lisays;
        }
        return palautus;
    }

    public boolean tilaa() {
        return tilaa > 0;
    }

    /**
     * Tulostetaan henkilön tiedot paramterina tuotuu tietovirtaan
     *
     * @param os tietovirta johon tiedot tulostetaan
     * @example <pre name="test">
     * #import java.io.*;
     * ByteArrayOutputStream byteoutput = new ByteArrayOutputStream();
     * Henkilo hlo = new Henkilo("Aku","Ankka",1934);
     * hlo.tulosta(byteoutput);
     * byteoutput.toString() =R= "Aku Ankka 1934\\s*"
     * </pre>
     */
    public void tulosta(OutputStream os) {
        PrintStream out = new PrintStream(os);
        out.printf("Paikat: %d, Matkustajat: %d, Vapaat: %d\n", paikat, matkustajat, tilaa);
    }

    @Override
    public String toString() {
        return paikat + "," + matkustajat + "," + tilaa;
    }

    public static void main(String[] args) {
        LinjaAuto pikkubussi = new LinjaAuto(10);
        LinjaAuto isobussi = new LinjaAuto(45);
        pikkubussi.lisaa(4);    pikkubussi.tulosta(System.out); // 10,4,6
        isobussi.lisaa(30);     isobussi.tulosta(System.out);   // 45,30,15
        int yli = pikkubussi.lisaa(15);
        isobussi.lisaa(yli);
        pikkubussi.tulosta(System.out);                         // 10,10,0
        isobussi.tulosta(System.out);                           // 45,39,6
        if ( pikkubussi.getTilaa() > 0 )
            System.out.println("Pieneen bussiin mahtuu!");   // ei tulosta
        if ( isobussi.tilaa() )
            System.out.println("Isoon bussiin mahtuu!");     // tulostaa
        int vajaa = pikkubussi.vahenna(12);                  // vajaa = -2
        if ( vajaa < 0 )
            System.out.println("Pikkubussissa ei edes ole näin montaa!");
        pikkubussi.tulosta(System.out);                         // 10,0,10
    }
}
