package d6;

import fi.jyu.mit.ohj2.Mjonot;

/**
 * PVM luokka
 * @author jokaliy
 * @version 1.0, 01.03.2025
 */

/**
 * @example
 * <pre name="test">
 *   Pvm pvm = new Pvm(20,2,2012);
 *   pvm.alusta(1,3,0);     pvm.toString() === "1.3.2012";
 *   pvm.alusta(2,13,2012); pvm.toString() === "1.3.2012";
 *   pvm.alusta(28,2,2012); pvm.toString() === "28.2.2012";
 *   pvm.alusta(29,2,2011); pvm.toString() === "28.2.2012";
 *   pvm.alusta(29,2,2012); pvm.toString() === "29.2.2012";
 *   pvm.alusta(31,3,2012); pvm.toString() === "31.3.2012";
 *   pvm.alusta(31,4,2012); pvm.toString() === "31.3.2012";
 *   pvm.alusta( 0,2,2012); pvm.toString() === "31.3.2012";
 * </pre>

 * @example
 * <pre name="test">
 *  Pvm pv1 = new Pvm(15,6,2013);
 *  Pvm pv2 = new Pvm(14,5,2014);
 *  Pvm pv3 = new Pvm(15,7,2014);
 *  Pvm pv4 = new Pvm(16,7,2014);
 *  Pvm pv5 = new Pvm(16,7,2014);
 *  Pvm pv6 = new Pvm(16,7,2012);
 *  Pvm.compareTo(pv1,pv2) === -1;  // ero vuodessa
 *  Pvm.compareTo(pv2,pv1) ===  1;
 *  Pvm.compareTo(pv2,pv3) === -1;  // ero kuukaudessa
 *  Pvm.compareTo(pv3,pv2) ===  1;
 *  Pvm.compareTo(pv3,pv4) === -1;  // ero päivässä
 *  Pvm.compareTo(pv4,pv3) ===  1;
 *  Pvm.compareTo(pv4,pv5) ===  0;  // kaikki samoja
 *  Pvm.compareTo(pv6,pv2) === -1;  // ero kuukaudessa, mutta vuodessa toisinpäin
 *  Pvm.compareTo(pv2,pv6) ===  1;
 * </pre>
 */
public class Pvm implements Comparable<Object> {
    private int pv, kk, vv;


    /**
     * Palauttaa tämänhetkisen päiväyksen.
     * TODO: muutettava toimimaan oikein.
     */
    public final void paivays() {
        pv = 30;
        kk = 1;
        vv = 2012;
    }


    /**
     * Alustetaan päivämäärä. 0-arvot eivät muuta vastaavaa attribuuttia
     *
     * @param ipv  päivän alustus
     * @param ikk  kuukauden alustus
     * @param iivv vuoden alustus
     */
    public final void alusta(final int ipv, final int ikk, final int iivv) {
        if (validoi(ipv > 0 ? ipv : this.pv, ikk > 0 ? ikk : this.kk, iivv > 0 ? iivv : this.vv)) {
            if (ipv > 0) this.pv = ipv;
            if (ikk > 0) this.kk = ikk;
            if (iivv > 0) this.vv = iivv;
            if (this.vv < 50) this.vv += 2000;
            if (this.vv < 100) this.vv += 1900;
        }

    }

    /**
     * Tarkistaa onko vuosi karkausvuosi
     * @param vuosi vuosiluku
     * @return true jos annettu vuosi on karkausvuosi
     */
    public static boolean onkoKarkausvuosi(final int vuosi) {
        if (vuosi % 4 == 0) {
            if (vuosi % 100 == 0) {
                return vuosi % 400 == 0;
            } else {
                return true;
            }
        }
        return false;
    }

    /**
     * Tarkistaa että päivä määrä on validi
     * @param ipv päivä
     * @param ikk kuukausi
     * @return palauttaa true, jos päivämäärä on validi
     *
     * @example <pre name="test">
     * Pvm pvm = new Pvm();
     * pvm.validoi(32,1, 2000) === false
     * pvm.validoi(20,4, 2000) === true
     * pvm.validoi(20,14, 2000) === false
     * </pre>
     */
    public boolean validoi(final int ipv, final int ikk, final int ivv) {
        //Validoidaan että kuukausi päivä ovat kohtuullisia arvoja
        if (ikk < 1 || ikk > 12  || ipv < 1 || ipv > 31) {
            return false;
        }
        //helmikuu
        if (ikk == 2) {
            return onkoKarkausvuosi(ivv) ? ipv < 30 : ipv < 29;
        }
        if (ipv == 31) {
            int[] kk31pv = {
                    1, 3, 5, 7, 8, 10, 12
            };
            for (int i : kk31pv) {
                if (ikk == i) {
                    return true;
                }
            }
            return false;
        }
        return true;
    }


    /**
     * Alustetaan kaikki attribuutit oletusarvoon
     */
    public Pvm() {
        this(0, 0, 0);
    }


    /**
     * Alustetaan kuukausi ja vuosi oletusarvoon
     *
     * @param pv päivän alustusarvo
     */
    public Pvm(int pv) {
        this(pv, 0, 0);
    }


    /**
     * Alustetaan vuosi oletusarvoon
     *
     * @param pv päivän alustusarvo
     * @param kk kuukauden oletusarvo
     */
    public Pvm(int pv, int kk) {
        this(pv, kk, 0);
    }


    /**
     * Alustetaan vuosi oletusarvoon
     *
     * @param pv päivän alustusarvo
     * @param kk kuukauden oletusarvo
     * @param vv vuoden alustusarvo
     */
    public Pvm(int pv, int kk, int vv) {
        paivays();
        alusta(pv, kk, vv);
    }


    /**
     * Palautetaan päiväys merkkijonona
     *
     * @return päiväys merkkijonoja
     * @example <pre name="test">
     * Pvm pvm = new Pvm(10,2,2008);
     * pvm.toString() === "10.2.2008";
     * </pre>
     */
    @Override
    public String toString() {
        return pv + "." + kk + "." + vv;
    }


    /**
     * Alustetaan päivän arvon merkkijonosta
     *
     * @param s jono josta pvm otetaan
     * @example <pre name="test">
     * Pvm pvm = new Pvm();
     * pvm.parse("11.05.02");  pvm.toString() === "11.5.2002";
     * pvm.parse("11.05.97");  pvm.toString() === "11.5.1997";
     * pvm.parse("17.3");      pvm.toString() === "17.3.1997";
     * pvm.parse("19");        pvm.toString() === "19.3.1997";
     * </pre>
     */
    public void parse(String s) {
        StringBuffer sb = new StringBuffer(s);
        int p = Mjonot.erota(sb, '.', 0);
        int k = Mjonot.erota(sb, '.', 0);
        int v = Mjonot.erota(sb, '.', 0);
        alusta(p, k, v);
    }

    /**
     * Vertailee pvm otusta annettuun pvm otukseen
     * @param o pvm otus
     * @return Jos pvm1 suurempi, palauttaa 1
     * Jos yhtäsuuret, palauttaa 0
     * Jos pvm2 suurempi, palauttaa -1
     *
     * @example <pre name="test">
     * Pvm pvm1 = new Pvm(2,2,2022);
     * Pvm pvm2 = new Pvm(6,2,2021);
     * pvm1.compareTo(pvm1, pvm2) === 1
     * pvm1.alusta(1,1, 2021);
     * pvm1.compareTo(pvm1, pvm2) === -1
     * pvm1.alusta(6,2, 2021);
     * pvm1.compareTo(pvm1, pvm2) === 0
     * </pre>
     */
    @Override
    public int compareTo(Object o) {
        Pvm pvm = (Pvm) o;
        // vertaillaan ensin vuosia
        if (this.vv > pvm.vv) {
            return 1;
        } else if (this.vv < pvm.vv) {
            return -1;
        }
        // sitten kuukausia
        if (this.kk > pvm.kk) {
            return 1;
        } else if (this.kk < pvm.kk) {
            return -1;
        }
        if (this.pv > pvm.pv) {
            return 1;
        } else if (this.pv < pvm.pv) {
            return -1;
        }
        return 0;
    }

    /**
     * Vertaa ovat päivämäärät yhtäsuuret
     * @param o päivämäärä otus
     * @return palauttaa true jos pvm otukset pv, kk, vv täsmäävät
     *
     * @example <pre name="test">
     * Pvm pvm1 = new Pvm(2,2,2022);
     * Pvm pvm2 = new Pvm(6,2,2021);
     * pvm1.equals(pvm2) === false
     * pvm1.alusta(6,2, 2021);
     * pvm1.equals(pvm2) === true
     * </pre>
     */
    @Override
    public boolean equals(Object o) {
        if (o instanceof Pvm pvm) {
            return this.compareTo(pvm) == 0;
        }
        return false;
    }

    /**
     * Vertailee kahta pvm otusta
     * @param pvm1 ensimmäinen pvm otus
     * @param pvm2 toinen pvm otus
     * @return Jos pvm1 suurempi, palauttaa 1
     * Jos yhtäsuuret, palauttaa 0
     * Jos pvm2 suurempi, palauttaa -1
     *
     * @example <pre name="test">
     * Pvm pvm1 = new Pvm(2,2,2022);
     * Pvm pvm2 = new Pvm(6,2,2021);
     * compareTo(pvm1, pvm2) === 1
     * pvm1.alusta(1,1, 2021);
     * compareTo(pvm1, pvm2) === -1
     * pvm1.alusta(6,2, 2021);
     * compareTo(pvm1, pvm2) === 0
     * </pre>
     */
    public static int compareTo(Pvm pvm1, Pvm pvm2) {
        // vertaillaan ensin vuosia
        if (pvm1.vv > pvm2.vv) {
            return 1;
        } else if (pvm1.vv < pvm2.vv) {
            return -1;
        }
        // sitten kuukausia
        if (pvm1.kk > pvm2.kk) {
            return 1;
        } else if (pvm1.kk < pvm2.kk) {
            return -1;
        }
        if (pvm1.pv > pvm2.pv) {
            return 1;
        } else if (pvm1.pv < pvm2.pv) {
            return -1;
        }
        return 0;
    }


    public static void main(String[] args) {}
}