package d9;

/**
 * @author Joonas Yli-Saari
 *
 * @example <pre name="test">
 * Ehdokas e = new Ehdokas(100, 5);
 * e.toString() === "Rahaa 100.00, ääniä 5"
 * </pre>
 */
public class Ehdokas {
    private double rahat;
    private int aanet;

    public Ehdokas(double rahat, int aanet) {
        this.rahat = rahat;
        this.aanet = aanet;
    }

    /**
     * Kasvattaa ehdokkaan äänien määrää mikäli rahat riittää
     * @param aanet äänien määrä
     * @param hinta hinta per ääni
     * @return true tai false mikäli onnistui tai ei
     *
     * @example <pre name="test">
     * Ehdokas e = new Ehdokas(100, 5);
     * e.osta(5, 5) === true
     * e.osta(5, 100) === false
     * </pre>
     */
    public boolean osta(int aanet, double hinta) {
        double kokonaishinta = aanet * hinta;
        if (kokonaishinta > rahat) {
            return false;
        }
        this.rahat -= kokonaishinta;
        this.aanet += aanet;
        return true;
    }


    public int compareTo(Ehdokas o) {
        if (this.aanet > o.aanet) {
            return 1;
        } else if (this.aanet < o.aanet) {
            return -1;
        } else {
            return 0;
        }
    }

    @Override
    public String toString() {
        return String.format("Rahaa %.2f, ääniä %d", rahat, aanet);
    }

    public static void main(String[] args)  {
        Ehdokas ehd1 = new Ehdokas(100000.0, 0);
        Ehdokas ehd2 = new Ehdokas(20000.0, 300);
        System.out.println(ehd1); // Tulostaa: Rahaa 100000.00, ääniä 0
        System.out.println(ehd2); // Tulostaa: Rahaa 20000.00, ääniä 300
        ehd1.osta(200, 100.0); // Ostaa 200 ääntä, 100 mk/kpl
        System.out.println(ehd1); // Tulostaa: Rahaa 80000.00, ääniä 200
        boolean onnistui = ehd2.osta(300, 100);
        if (!onnistui) System.out.println("Rahat ei riitä :-)");
        System.out.println(ehd2); // Tulostaa: Rahaa 20000.00, ääniä 300
        if (ehd1.compareTo(ehd2) > 0) System.out.println("Ehdokas 1 voitti!");
        if (ehd1.compareTo(ehd2) < 0) System.out.println("Ehdokas 2 voitti!");
        if (ehd1.compareTo(ehd2) == 0) System.out.println("Tasapeli!");
        // Vertailu tehdään äänimäärien perusteella.
        // Esimerkissä tulostuu : Ehdokas 2 voitti!
    }

}
