package d9;

public class Taulukko {
    /**
     * Luokka tyden taulukon poikkeusta varten
     */
    public static class TaulukkoTaysiException extends Exception {
        private static final long serialVersionUID = 1L;


        TaulukkoTaysiException(String viesti) {
            super(viesti);
        }
    }

    private int alkiot[];
    private int lkm;


    /**
     * Alustetaan 5 kokoinen taulukko
     */
    public Taulukko() {
        alkiot = new int[5];
    }


    /**
     * Alutetaan valitun kokoinen taulukko
     *
     * @param koko taulukon koko
     */
    public Taulukko(int koko) {
        alkiot = new int[koko];
    }


    /**
     * Listn taulukkoon yksi alkio
     *
     * @param luku listtv alkio
     * @throws TaulukkoTaysiException jos taulukko tysi
     */
    public void lisaa(int luku) throws TaulukkoTaysiException {
        if (lkm >= alkiot.length) throw new TaulukkoTaysiException("Tila loppu");
        alkiot[lkm++] = luku;
    }

    /**
     * @param luku poistaa kaikki luvut esiintymät taulukosta
     * @example <pre name="test">
     *     #THROWS Taulukko.TaulukkoTaysiException
     *        Taulukko luvut = new Taulukko(7);
     *         luvut.lisaa(1); luvut.lisaa(2); luvut.lisaa(1);
     *         luvut.toString() === "1,2,1"
     *         luvut.poista(1);
     *         luvut.toString()  === "2"
     * </pre>
     */
    public void poista(int luku) {
        int poistot = 0;
        for (int i = 0; i < lkm; i++) {
            if (alkiot[i] == luku) {
                poistot++;
            } else {
                alkiot[i - poistot] = alkiot[i];
            }
        }
        lkm -= poistot;
    }


    @Override
    public String toString() {
        StringBuilder s = new StringBuilder("");
        String erotin = "";
        for (int i = 0; i < lkm; i++) {
            s.append(erotin + alkiot[i]);
            erotin = ",";
        }
        return s.toString();
    }


    /**
     * Asetetaan taulukon i's alkio
     *
     * @param i    mihin paikkaan astetaan
     * @param luku mik arvo laitetaan
     * @throws IndexOutOfBoundsException jos vr indeksi
     */
    public void set(int i, int luku) throws IndexOutOfBoundsException {
        if ((i < 0) || (lkm <= i)) throw new IndexOutOfBoundsException("i = " + i);
        alkiot[i] = luku;
    }


    /**
     * Palautetana paikasssa i oleva luku
     *
     * @param i mist paikasta luku otetaan
     * @return paikassa i olevan luvun arvo
     * @throws IndexOutOfBoundsException jos indeksi vrin
     */
    public int get(int i) throws IndexOutOfBoundsException {
        if ((i < 0) || (lkm <= i)) throw new IndexOutOfBoundsException("i = " + i);
        return alkiot[i];
    }

    @Override
    public Taulukko clone() {
        Taulukko t1 = new Taulukko(alkiot.length);
        t1.lkm = lkm;
        t1.alkiot = this.alkiot.clone();
        return t1;
    }

}

